"""Wrapper for RGVS, RGSF, and various other functions"""

__version__ = '1.7.0'

__author_name__, __author_email__ = 'Irrational Encoding Wizardry', 'wizards@encode.moe'
__maintainer_name__, __maintainer_email__ = 'Setsugen no ao', 'setsugen@setsugen.dev'

__author__ = f'{__author_name__} <{__author_email__}>'
__maintainer__ = __author__

if __name__ == '__github__':
    print(__version__)
